"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .outputitemimagegenerationcall import (
    OutputItemImageGenerationCall,
    OutputItemImageGenerationCallTypedDict,
)
from .outputmessage import OutputMessage, OutputMessageTypedDict
from .responseinputaudio import ResponseInputAudio, ResponseInputAudioTypedDict
from .responseinputfile import ResponseInputFile, ResponseInputFileTypedDict
from .responseinputimage import ResponseInputImage, ResponseInputImageTypedDict
from .responseinputtext import ResponseInputText, ResponseInputTextTypedDict
from .toolcallstatus import ToolCallStatus
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import get_discriminator, validate_open_enum
from pydantic import Discriminator, Tag, model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OpenAIResponsesInputTypeFunctionCall = Literal["function_call",]


class OpenAIResponsesInputFunctionCallTypedDict(TypedDict):
    type: OpenAIResponsesInputTypeFunctionCall
    call_id: str
    name: str
    arguments: str
    id: NotRequired[str]
    status: NotRequired[Nullable[ToolCallStatus]]


class OpenAIResponsesInputFunctionCall(BaseModel):
    type: OpenAIResponsesInputTypeFunctionCall

    call_id: str

    name: str

    arguments: str

    id: Optional[str] = None

    status: Annotated[
        OptionalNullable[ToolCallStatus], PlainValidator(validate_open_enum(False))
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "status"]
        nullable_fields = ["status"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenAIResponsesInputTypeFunctionCallOutput = Literal["function_call_output",]


class OpenAIResponsesInputFunctionCallOutputTypedDict(TypedDict):
    type: OpenAIResponsesInputTypeFunctionCallOutput
    call_id: str
    output: str
    id: NotRequired[Nullable[str]]
    status: NotRequired[Nullable[ToolCallStatus]]


class OpenAIResponsesInputFunctionCallOutput(BaseModel):
    type: OpenAIResponsesInputTypeFunctionCallOutput

    call_id: str

    output: str

    id: OptionalNullable[str] = UNSET

    status: Annotated[
        OptionalNullable[ToolCallStatus], PlainValidator(validate_open_enum(False))
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "status"]
        nullable_fields = ["id", "status"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenAIResponsesInputTypeMessage2 = Literal["message",]


OpenAIResponsesInputRoleDeveloper2 = Literal["developer",]


OpenAIResponsesInputRoleSystem2 = Literal["system",]


OpenAIResponsesInputRoleUser2 = Literal["user",]


OpenAIResponsesInputRoleUnion2TypedDict = TypeAliasType(
    "OpenAIResponsesInputRoleUnion2TypedDict",
    Union[
        OpenAIResponsesInputRoleUser2,
        OpenAIResponsesInputRoleSystem2,
        OpenAIResponsesInputRoleDeveloper2,
    ],
)


OpenAIResponsesInputRoleUnion2 = TypeAliasType(
    "OpenAIResponsesInputRoleUnion2",
    Union[
        OpenAIResponsesInputRoleUser2,
        OpenAIResponsesInputRoleSystem2,
        OpenAIResponsesInputRoleDeveloper2,
    ],
)


OpenAIResponsesInputContent3TypedDict = TypeAliasType(
    "OpenAIResponsesInputContent3TypedDict",
    Union[
        ResponseInputTextTypedDict,
        ResponseInputAudioTypedDict,
        ResponseInputImageTypedDict,
        ResponseInputFileTypedDict,
    ],
)


OpenAIResponsesInputContent3 = Annotated[
    Union[
        Annotated[ResponseInputText, Tag("input_text")],
        Annotated[ResponseInputImage, Tag("input_image")],
        Annotated[ResponseInputFile, Tag("input_file")],
        Annotated[ResponseInputAudio, Tag("input_audio")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class OpenAIResponsesInputMessage2TypedDict(TypedDict):
    id: str
    role: OpenAIResponsesInputRoleUnion2TypedDict
    content: List[OpenAIResponsesInputContent3TypedDict]
    type: NotRequired[OpenAIResponsesInputTypeMessage2]


class OpenAIResponsesInputMessage2(BaseModel):
    id: str

    role: OpenAIResponsesInputRoleUnion2

    content: List[OpenAIResponsesInputContent3]

    type: Optional[OpenAIResponsesInputTypeMessage2] = None


OpenAIResponsesInputTypeMessage1 = Literal["message",]


OpenAIResponsesInputRoleDeveloper1 = Literal["developer",]


OpenAIResponsesInputRoleAssistant = Literal["assistant",]


OpenAIResponsesInputRoleSystem1 = Literal["system",]


OpenAIResponsesInputRoleUser1 = Literal["user",]


OpenAIResponsesInputRoleUnion1TypedDict = TypeAliasType(
    "OpenAIResponsesInputRoleUnion1TypedDict",
    Union[
        OpenAIResponsesInputRoleUser1,
        OpenAIResponsesInputRoleSystem1,
        OpenAIResponsesInputRoleAssistant,
        OpenAIResponsesInputRoleDeveloper1,
    ],
)


OpenAIResponsesInputRoleUnion1 = TypeAliasType(
    "OpenAIResponsesInputRoleUnion1",
    Union[
        OpenAIResponsesInputRoleUser1,
        OpenAIResponsesInputRoleSystem1,
        OpenAIResponsesInputRoleAssistant,
        OpenAIResponsesInputRoleDeveloper1,
    ],
)


OpenAIResponsesInputContent1TypedDict = TypeAliasType(
    "OpenAIResponsesInputContent1TypedDict",
    Union[
        ResponseInputTextTypedDict,
        ResponseInputAudioTypedDict,
        ResponseInputImageTypedDict,
        ResponseInputFileTypedDict,
    ],
)


OpenAIResponsesInputContent1 = Annotated[
    Union[
        Annotated[ResponseInputText, Tag("input_text")],
        Annotated[ResponseInputImage, Tag("input_image")],
        Annotated[ResponseInputFile, Tag("input_file")],
        Annotated[ResponseInputAudio, Tag("input_audio")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


OpenAIResponsesInputContent2TypedDict = TypeAliasType(
    "OpenAIResponsesInputContent2TypedDict",
    Union[List[OpenAIResponsesInputContent1TypedDict], str],
)


OpenAIResponsesInputContent2 = TypeAliasType(
    "OpenAIResponsesInputContent2", Union[List[OpenAIResponsesInputContent1], str]
)


class OpenAIResponsesInputMessage1TypedDict(TypedDict):
    role: OpenAIResponsesInputRoleUnion1TypedDict
    content: OpenAIResponsesInputContent2TypedDict
    type: NotRequired[OpenAIResponsesInputTypeMessage1]


class OpenAIResponsesInputMessage1(BaseModel):
    role: OpenAIResponsesInputRoleUnion1

    content: OpenAIResponsesInputContent2

    type: Optional[OpenAIResponsesInputTypeMessage1] = None


OpenAIResponsesInputUnion1TypedDict = TypeAliasType(
    "OpenAIResponsesInputUnion1TypedDict",
    Union[
        OpenAIResponsesInputMessage1TypedDict,
        OpenAIResponsesInputMessage2TypedDict,
        OutputItemImageGenerationCallTypedDict,
        OpenAIResponsesInputFunctionCallOutputTypedDict,
        OutputMessageTypedDict,
        OpenAIResponsesInputFunctionCallTypedDict,
    ],
)


OpenAIResponsesInputUnion1 = TypeAliasType(
    "OpenAIResponsesInputUnion1",
    Union[
        OpenAIResponsesInputMessage1,
        OpenAIResponsesInputMessage2,
        OutputItemImageGenerationCall,
        OpenAIResponsesInputFunctionCallOutput,
        OutputMessage,
        OpenAIResponsesInputFunctionCall,
    ],
)


OpenAIResponsesInputUnionTypedDict = TypeAliasType(
    "OpenAIResponsesInputUnionTypedDict",
    Union[str, List[OpenAIResponsesInputUnion1TypedDict], Any],
)


OpenAIResponsesInputUnion = TypeAliasType(
    "OpenAIResponsesInputUnion", Union[str, List[OpenAIResponsesInputUnion1], Any]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responsesformatjsonobject import (
    ResponsesFormatJSONObject,
    ResponsesFormatJSONObjectTypedDict,
)
from .responsesformattext import ResponsesFormatText, ResponsesFormatTextTypedDict
from .responsesformattextjsonschemaconfig import (
    ResponsesFormatTextJSONSchemaConfig,
    ResponsesFormatTextJSONSchemaConfigTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ResponseFormatTextConfigTypedDict = TypeAliasType(
    "ResponseFormatTextConfigTypedDict",
    Union[
        ResponsesFormatTextTypedDict,
        ResponsesFormatJSONObjectTypedDict,
        ResponsesFormatTextJSONSchemaConfigTypedDict,
    ],
)
r"""Text response format configuration"""


ResponseFormatTextConfig = TypeAliasType(
    "ResponseFormatTextConfig",
    Union[
        ResponsesFormatText,
        ResponsesFormatJSONObject,
        ResponsesFormatTextJSONSchemaConfig,
    ],
)
r"""Text response format configuration"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal, Union
from typing_extensions import TypeAliasType, TypedDict


OpenAIResponsesToolChoiceTypeWebSearchPreview = Literal["web_search_preview",]


OpenAIResponsesToolChoiceTypeWebSearchPreview20250311 = Literal[
    "web_search_preview_2025_03_11",
]


TypeTypedDict = TypeAliasType(
    "TypeTypedDict",
    Union[
        OpenAIResponsesToolChoiceTypeWebSearchPreview20250311,
        OpenAIResponsesToolChoiceTypeWebSearchPreview,
    ],
)


Type = TypeAliasType(
    "Type",
    Union[
        OpenAIResponsesToolChoiceTypeWebSearchPreview20250311,
        OpenAIResponsesToolChoiceTypeWebSearchPreview,
    ],
)


class OpenAIResponsesToolChoiceTypedDict(TypedDict):
    type: TypeTypedDict


class OpenAIResponsesToolChoice(BaseModel):
    type: Type


OpenAIResponsesToolChoiceTypeFunction = Literal["function",]


class OpenAIResponsesToolChoiceFunctionTypedDict(TypedDict):
    type: OpenAIResponsesToolChoiceTypeFunction
    name: str


class OpenAIResponsesToolChoiceFunction(BaseModel):
    type: OpenAIResponsesToolChoiceTypeFunction

    name: str


OpenAIResponsesToolChoiceRequired = Literal["required",]


OpenAIResponsesToolChoiceNone = Literal["none",]


OpenAIResponsesToolChoiceAuto = Literal["auto",]


OpenAIResponsesToolChoiceUnionTypedDict = TypeAliasType(
    "OpenAIResponsesToolChoiceUnionTypedDict",
    Union[
        OpenAIResponsesToolChoiceTypedDict,
        OpenAIResponsesToolChoiceFunctionTypedDict,
        OpenAIResponsesToolChoiceAuto,
        OpenAIResponsesToolChoiceNone,
        OpenAIResponsesToolChoiceRequired,
    ],
)


OpenAIResponsesToolChoiceUnion = TypeAliasType(
    "OpenAIResponsesToolChoiceUnion",
    Union[
        OpenAIResponsesToolChoice,
        OpenAIResponsesToolChoiceFunction,
        OpenAIResponsesToolChoiceAuto,
        OpenAIResponsesToolChoiceNone,
        OpenAIResponsesToolChoiceRequired,
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


OpenAIResponsesReasoningEffort = Union[
    Literal[
        "high",
        "medium",
        "low",
        "minimal",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .toolcallstatus import ToolCallStatus
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal
from typing_extensions import Annotated, NotRequired, TypedDict


OpenResponsesFunctionToolCallType = Literal["function_call",]


class OpenResponsesFunctionToolCallTypedDict(TypedDict):
    r"""A function call initiated by the model"""

    type: OpenResponsesFunctionToolCallType
    call_id: str
    name: str
    arguments: str
    id: str
    status: NotRequired[Nullable[ToolCallStatus]]


class OpenResponsesFunctionToolCall(BaseModel):
    r"""A function call initiated by the model"""

    type: OpenResponsesFunctionToolCallType

    call_id: str

    name: str

    arguments: str

    id: str

    status: Annotated[
        OptionalNullable[ToolCallStatus], PlainValidator(validate_open_enum(False))
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["status"]
        nullable_fields = ["status"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from datetime import datetime
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict, deprecated


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class RateLimitTypedDict(TypedDict):
    r"""Legacy rate limit information about a key. Will always return -1."""

    requests: float
    r"""Number of requests allowed per interval"""
    interval: str
    r"""Rate limit interval"""
    note: str
    r"""Note about the rate limit"""


@deprecated(
    "warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
)
class RateLimit(BaseModel):
    r"""Legacy rate limit information about a key. Will always return -1."""

    requests: float
    r"""Number of requests allowed per interval"""

    interval: str
    r"""Rate limit interval"""

    note: str
    r"""Note about the rate limit"""


class GetCurrentKeyDataTypedDict(TypedDict):
    r"""Current API key information"""

    label: str
    r"""Human-readable label for the API key"""
    limit: Nullable[float]
    r"""Spending limit for the API key in USD"""
    usage: float
    r"""Total OpenRouter credit usage (in USD) for the API key"""
    usage_daily: float
    r"""OpenRouter credit usage (in USD) for the current UTC day"""
    usage_weekly: float
    r"""OpenRouter credit usage (in USD) for the current UTC week (Monday-Sunday)"""
    usage_monthly: float
    r"""OpenRouter credit usage (in USD) for the current UTC month"""
    byok_usage: float
    r"""Total external BYOK usage (in USD) for the API key"""
    byok_usage_daily: float
    r"""External BYOK usage (in USD) for the current UTC day"""
    byok_usage_weekly: float
    r"""External BYOK usage (in USD) for the current UTC week (Monday-Sunday)"""
    byok_usage_monthly: float
    r"""External BYOK usage (in USD) for current UTC month"""
    is_free_tier: bool
    r"""Whether this is a free tier API key"""
    is_provisioning_key: bool
    r"""Whether this is a provisioning key"""
    limit_remaining: Nullable[float]
    r"""Remaining spending limit in USD"""
    limit_reset: Nullable[str]
    r"""Type of limit reset for the API key"""
    include_byok_in_limit: bool
    r"""Whether to include external BYOK usage in the credit limit"""
    rate_limit: RateLimitTypedDict
    r"""Legacy rate limit information about a key. Will always return -1."""
    expires_at: NotRequired[Nullable[datetime]]
    r"""ISO 8601 UTC timestamp when the API key expires, or null if no expiration"""


class GetCurrentKeyData(BaseModel):
    r"""Current API key information"""

    label: str
    r"""Human-readable label for the API key"""

    limit: Nullable[float]
    r"""Spending limit for the API key in USD"""

    usage: float
    r"""Total OpenRouter credit usage (in USD) for the API key"""

    usage_daily: float
    r"""OpenRouter credit usage (in USD) for the current UTC day"""

    usage_weekly: float
    r"""OpenRouter credit usage (in USD) for the current UTC week (Monday-Sunday)"""

    usage_monthly: float
    r"""OpenRouter credit usage (in USD) for the current UTC month"""

    byok_usage: float
    r"""Total external BYOK usage (in USD) for the API key"""

    byok_usage_daily: float
    r"""External BYOK usage (in USD) for the current UTC day"""

    byok_usage_weekly: float
    r"""External BYOK usage (in USD) for the current UTC week (Monday-Sunday)"""

    byok_usage_monthly: float
    r"""External BYOK usage (in USD) for current UTC month"""

    is_free_tier: bool
    r"""Whether this is a free tier API key"""

    is_provisioning_key: bool
    r"""Whether this is a provisioning key"""

    limit_remaining: Nullable[float]
    r"""Remaining spending limit in USD"""

    limit_reset: Nullable[str]
    r"""Type of limit reset for the API key"""

    include_byok_in_limit: bool
    r"""Whether to include external BYOK usage in the credit limit"""

    rate_limit: Annotated[
        RateLimit,
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ]
    r"""Legacy rate limit information about a key. Will always return -1."""

    expires_at: OptionalNullable[datetime] = UNSET
    r"""ISO 8601 UTC timestamp when the API key expires, or null if no expiration"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["expires_at"]
        nullable_fields = ["limit", "limit_remaining", "limit_reset", "expires_at"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class GetCurrentKeyResponseTypedDict(TypedDict):
    r"""API key details"""

    data: GetCurrentKeyDataTypedDict
    r"""Current API key information"""


class GetCurrentKeyResponse(BaseModel):
    r"""API key details"""

    data: GetCurrentKeyData
    r"""Current API key information"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from openrouter import components, errors, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Mapping, Optional, Union


class Completions(BaseSDK):
    def generate(
        self,
        *,
        prompt: Union[components.Prompt, components.PromptTypedDict],
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        best_of: OptionalNullable[int] = UNSET,
        echo: OptionalNullable[bool] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[int] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        n: OptionalNullable[int] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.CompletionCreateParamsStop,
                components.CompletionCreateParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Optional[bool] = False,
        stream_options: OptionalNullable[
            Union[components.StreamOptions, components.StreamOptionsTypedDict]
        ] = UNSET,
        suffix: OptionalNullable[str] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        user: Optional[str] = None,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        response_format: OptionalNullable[
            Union[
                components.CompletionCreateParamsResponseFormatUnion,
                components.CompletionCreateParamsResponseFormatUnionTypedDict,
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.CompletionResponse:
        r"""Create a completion

        Creates a completion for the provided prompt and parameters. Supports both streaming and non-streaming modes.

        :param prompt:
        :param model:
        :param models:
        :param best_of:
        :param echo:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param max_tokens:
        :param n:
        :param presence_penalty:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param suffix:
        :param temperature:
        :param top_p:
        :param user:
        :param metadata:
        :param response_format:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.CompletionCreateParams(
            model=model,
            models=models,
            prompt=prompt,
            best_of=best_of,
            echo=echo,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            max_tokens=max_tokens,
            n=n,
            presence_penalty=presence_penalty,
            seed=seed,
            stop=stop,
            stream=stream,
            stream_options=utils.get_pydantic_model(
                stream_options, OptionalNullable[components.StreamOptions]
            ),
            suffix=suffix,
            temperature=temperature,
            top_p=top_p,
            user=user,
            metadata=metadata,
            response_format=utils.get_pydantic_model(
                response_format,
                OptionalNullable[components.CompletionCreateParamsResponseFormatUnion],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.CompletionCreateParams
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCompletions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(components.CompletionResponse, http_res)
        if utils.match_response(http_res, ["400", "401", "429"], "application/json"):
            response_data = unmarshal_json_response(errors.ChatErrorData, http_res)
            raise errors.ChatError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ChatErrorData, http_res)
            raise errors.ChatError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def generate_async(
        self,
        *,
        prompt: Union[components.Prompt, components.PromptTypedDict],
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        best_of: OptionalNullable[int] = UNSET,
        echo: OptionalNullable[bool] = UNSET,
        frequency_penalty: OptionalNullable[float] = UNSET,
        logit_bias: OptionalNullable[Dict[str, float]] = UNSET,
        logprobs: OptionalNullable[int] = UNSET,
        max_tokens: OptionalNullable[int] = UNSET,
        n: OptionalNullable[int] = UNSET,
        presence_penalty: OptionalNullable[float] = UNSET,
        seed: OptionalNullable[int] = UNSET,
        stop: OptionalNullable[
            Union[
                components.CompletionCreateParamsStop,
                components.CompletionCreateParamsStopTypedDict,
            ]
        ] = UNSET,
        stream: Optional[bool] = False,
        stream_options: OptionalNullable[
            Union[components.StreamOptions, components.StreamOptionsTypedDict]
        ] = UNSET,
        suffix: OptionalNullable[str] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        user: Optional[str] = None,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        response_format: OptionalNullable[
            Union[
                components.CompletionCreateParamsResponseFormatUnion,
                components.CompletionCreateParamsResponseFormatUnionTypedDict,
            ]
        ] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.CompletionResponse:
        r"""Create a completion

        Creates a completion for the provided prompt and parameters. Supports both streaming and non-streaming modes.

        :param prompt:
        :param model:
        :param models:
        :param best_of:
        :param echo:
        :param frequency_penalty:
        :param logit_bias:
        :param logprobs:
        :param max_tokens:
        :param n:
        :param presence_penalty:
        :param seed:
        :param stop:
        :param stream:
        :param stream_options:
        :param suffix:
        :param temperature:
        :param top_p:
        :param user:
        :param metadata:
        :param response_format:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.CompletionCreateParams(
            model=model,
            models=models,
            prompt=prompt,
            best_of=best_of,
            echo=echo,
            frequency_penalty=frequency_penalty,
            logit_bias=logit_bias,
            logprobs=logprobs,
            max_tokens=max_tokens,
            n=n,
            presence_penalty=presence_penalty,
            seed=seed,
            stop=stop,
            stream=stream,
            stream_options=utils.get_pydantic_model(
                stream_options, OptionalNullable[components.StreamOptions]
            ),
            suffix=suffix,
            temperature=temperature,
            top_p=top_p,
            user=user,
            metadata=metadata,
            response_format=utils.get_pydantic_model(
                response_format,
                OptionalNullable[components.CompletionCreateParamsResponseFormatUnion],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/completions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.CompletionCreateParams
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createCompletions",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(components.CompletionResponse, http_res)
        if utils.match_response(http_res, ["400", "401", "429"], "application/json"):
            response_data = unmarshal_json_response(errors.ChatErrorData, http_res)
            raise errors.ChatError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(errors.ChatErrorData, http_res)
            raise errors.ChatError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing_extensions import TypedDict


class GetCreditsResponseTypedDict(TypedDict):
    r"""Total credits purchased and used"""


class GetCreditsResponse(BaseModel):
    r"""Total credits purchased and used"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


Reason = Union[
    Literal[
        "max_output_tokens",
        "content_filter",
    ],
    UnrecognizedStr,
]


class OpenAIResponsesIncompleteDetailsTypedDict(TypedDict):
    reason: NotRequired[Reason]


class OpenAIResponsesIncompleteDetails(BaseModel):
    reason: Annotated[Optional[Reason], PlainValidator(validate_open_enum(False))] = (
        None
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


FilePathType = Literal["file_path",]


class FilePathTypedDict(TypedDict):
    type: FilePathType
    file_id: str
    index: float


class FilePath(BaseModel):
    type: FilePathType

    file_id: str

    index: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesincludable import OpenAIResponsesIncludable
from .openairesponsesprompt import OpenAIResponsesPrompt, OpenAIResponsesPromptTypedDict
from .openairesponsestoolchoice_union import (
    OpenAIResponsesToolChoiceUnion,
    OpenAIResponsesToolChoiceUnionTypedDict,
)
from .openresponsesinput import OpenResponsesInput, OpenResponsesInputTypedDict
from .openresponsesreasoningconfig import (
    OpenResponsesReasoningConfig,
    OpenResponsesReasoningConfigTypedDict,
)
from .openresponsesresponsetext import (
    OpenResponsesResponseText,
    OpenResponsesResponseTextTypedDict,
)
from .openresponseswebsearch20250826tool import (
    OpenResponsesWebSearch20250826Tool,
    OpenResponsesWebSearch20250826ToolTypedDict,
)
from .openresponseswebsearchpreview20250311tool import (
    OpenResponsesWebSearchPreview20250311Tool,
    OpenResponsesWebSearchPreview20250311ToolTypedDict,
)
from .openresponseswebsearchpreviewtool import (
    OpenResponsesWebSearchPreviewTool,
    OpenResponsesWebSearchPreviewToolTypedDict,
)
from .openresponseswebsearchtool import (
    OpenResponsesWebSearchTool,
    OpenResponsesWebSearchToolTypedDict,
)
from .providername import ProviderName
from .quantization import Quantization
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OpenResponsesRequestType = Literal["function",]


class OpenResponsesRequestToolFunctionTypedDict(TypedDict):
    r"""Function tool definition"""

    type: OpenResponsesRequestType
    name: str
    parameters: Nullable[Dict[str, Nullable[Any]]]
    description: NotRequired[Nullable[str]]
    strict: NotRequired[Nullable[bool]]


class OpenResponsesRequestToolFunction(BaseModel):
    r"""Function tool definition"""

    type: OpenResponsesRequestType

    name: str

    parameters: Nullable[Dict[str, Nullable[Any]]]

    description: OptionalNullable[str] = UNSET

    strict: OptionalNullable[bool] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "strict"]
        nullable_fields = ["description", "strict", "parameters"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenResponsesRequestToolUnionTypedDict = TypeAliasType(
    "OpenResponsesRequestToolUnionTypedDict",
    Union[
        OpenResponsesWebSearchPreviewToolTypedDict,
        OpenResponsesWebSearchPreview20250311ToolTypedDict,
        OpenResponsesWebSearchToolTypedDict,
        OpenResponsesWebSearch20250826ToolTypedDict,
        OpenResponsesRequestToolFunctionTypedDict,
    ],
)


OpenResponsesRequestToolUnion = TypeAliasType(
    "OpenResponsesRequestToolUnion",
    Union[
        OpenResponsesWebSearchPreviewTool,
        OpenResponsesWebSearchPreview20250311Tool,
        OpenResponsesWebSearchTool,
        OpenResponsesWebSearch20250826Tool,
        OpenResponsesRequestToolFunction,
    ],
)


ServiceTier = Union[
    Literal[
        "auto",
        "default",
        "flex",
        "priority",
        "scale",
    ],
    UnrecognizedStr,
]


Truncation = Union[
    Literal[
        "auto",
        "disabled",
    ],
    UnrecognizedStr,
]


DataCollection = Union[
    Literal[
        "deny",
        "allow",
    ],
    UnrecognizedStr,
]
r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
- allow: (default) allow providers which store user data non-transiently and may train on it
- deny: use only providers which do not collect user data.

"""


OrderTypedDict = TypeAliasType("OrderTypedDict", Union[ProviderName, str])


Order = TypeAliasType(
    "Order",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


OnlyTypedDict = TypeAliasType("OnlyTypedDict", Union[ProviderName, str])


Only = TypeAliasType(
    "Only",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


IgnoreTypedDict = TypeAliasType("IgnoreTypedDict", Union[ProviderName, str])


Ignore = TypeAliasType(
    "Ignore",
    Union[Annotated[ProviderName, PlainValidator(validate_open_enum(False))], str],
)


Sort = Union[
    Literal[
        "price",
        "throughput",
        "latency",
    ],
    UnrecognizedStr,
]
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


class MaxPriceTypedDict(TypedDict):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    completion: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    image: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    audio: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    request: NotRequired[Any]
    r"""A value in string or number format that is a large number"""


class MaxPrice(BaseModel):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    completion: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    image: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    audio: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    request: Optional[Any] = None
    r"""A value in string or number format that is a large number"""


class ProviderTypedDict(TypedDict):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: NotRequired[Nullable[bool]]
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """
    require_parameters: NotRequired[Nullable[bool]]
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""
    data_collection: NotRequired[Nullable[DataCollection]]
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it
    - deny: use only providers which do not collect user data.

    """
    zdr: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""
    enforce_distillable_text: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""
    order: NotRequired[Nullable[List[OrderTypedDict]]]
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""
    only: NotRequired[Nullable[List[OnlyTypedDict]]]
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""
    ignore: NotRequired[Nullable[List[IgnoreTypedDict]]]
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""
    quantizations: NotRequired[Nullable[List[Quantization]]]
    r"""A list of quantization levels to filter the provider by."""
    sort: NotRequired[Nullable[Sort]]
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""
    max_price: NotRequired[MaxPriceTypedDict]
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""


class Provider(BaseModel):
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    allow_fallbacks: OptionalNullable[bool] = UNSET
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """

    require_parameters: OptionalNullable[bool] = UNSET
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""

    data_collection: Annotated[
        OptionalNullable[DataCollection], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it
    - deny: use only providers which do not collect user data.

    """

    zdr: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""

    enforce_distillable_text: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""

    order: OptionalNullable[List[Order]] = UNSET
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""

    only: OptionalNullable[List[Only]] = UNSET
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""

    ignore: OptionalNullable[List[Ignore]] = UNSET
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""

    quantizations: OptionalNullable[
        List[Annotated[Quantization, PlainValidator(validate_open_enum(False))]]
    ] = UNSET
    r"""A list of quantization levels to filter the provider by."""

    sort: Annotated[
        OptionalNullable[Sort], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""

    max_price: Optional[MaxPrice] = None
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "max_price",
        ]
        nullable_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


IDFileParser = Literal["file-parser",]


PdfEngine = Union[
    Literal[
        "mistral-ocr",
        "pdf-text",
        "native",
    ],
    UnrecognizedStr,
]


class PdfTypedDict(TypedDict):
    engine: NotRequired[PdfEngine]


class Pdf(BaseModel):
    engine: Annotated[
        Optional[PdfEngine], PlainValidator(validate_open_enum(False))
    ] = None


class PluginFileParserTypedDict(TypedDict):
    id: IDFileParser
    max_files: NotRequired[float]
    pdf: NotRequired[PdfTypedDict]


class PluginFileParser(BaseModel):
    id: IDFileParser

    max_files: Optional[float] = None

    pdf: Optional[Pdf] = None


IDWeb = Literal["web",]


Engine = Union[
    Literal[
        "native",
        "exa",
    ],
    UnrecognizedStr,
]


class PluginWebTypedDict(TypedDict):
    id: IDWeb
    max_results: NotRequired[float]
    search_prompt: NotRequired[str]
    engine: NotRequired[Engine]


class PluginWeb(BaseModel):
    id: IDWeb

    max_results: Optional[float] = None

    search_prompt: Optional[str] = None

    engine: Annotated[Optional[Engine], PlainValidator(validate_open_enum(False))] = (
        None
    )


IDModeration = Literal["moderation",]


class PluginModerationTypedDict(TypedDict):
    id: IDModeration


class PluginModeration(BaseModel):
    id: IDModeration


PluginTypedDict = TypeAliasType(
    "PluginTypedDict",
    Union[PluginModerationTypedDict, PluginFileParserTypedDict, PluginWebTypedDict],
)


Plugin = TypeAliasType("Plugin", Union[PluginModeration, PluginFileParser, PluginWeb])


class OpenResponsesRequestTypedDict(TypedDict):
    r"""Request schema for Responses endpoint"""

    input: NotRequired[OpenResponsesInputTypedDict]
    r"""Input for a response request - can be a string or array of items"""
    instructions: NotRequired[Nullable[str]]
    metadata: NotRequired[Nullable[Dict[str, str]]]
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""
    tools: NotRequired[List[OpenResponsesRequestToolUnionTypedDict]]
    tool_choice: NotRequired[OpenAIResponsesToolChoiceUnionTypedDict]
    parallel_tool_calls: NotRequired[Nullable[bool]]
    model: NotRequired[str]
    models: NotRequired[List[str]]
    text: NotRequired[OpenResponsesResponseTextTypedDict]
    r"""Text output configuration including format and verbosity"""
    reasoning: NotRequired[Nullable[OpenResponsesReasoningConfigTypedDict]]
    r"""Configuration for reasoning mode in the response"""
    max_output_tokens: NotRequired[Nullable[float]]
    temperature: NotRequired[Nullable[float]]
    top_p: NotRequired[Nullable[float]]
    top_k: NotRequired[float]
    prompt_cache_key: NotRequired[Nullable[str]]
    previous_response_id: NotRequired[Nullable[str]]
    prompt: NotRequired[Nullable[OpenAIResponsesPromptTypedDict]]
    include: NotRequired[Nullable[List[OpenAIResponsesIncludable]]]
    background: NotRequired[Nullable[bool]]
    safety_identifier: NotRequired[Nullable[str]]
    store: NotRequired[Nullable[bool]]
    service_tier: NotRequired[Nullable[ServiceTier]]
    truncation: NotRequired[Nullable[Truncation]]
    stream: NotRequired[bool]
    provider: NotRequired[Nullable[ProviderTypedDict]]
    r"""When multiple model providers are available, optionally indicate your routing preference."""
    plugins: NotRequired[List[PluginTypedDict]]
    r"""Plugins you want to enable for this request, including their settings."""
    user: NotRequired[str]
    r"""A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters."""


class OpenResponsesRequest(BaseModel):
    r"""Request schema for Responses endpoint"""

    input: Optional[OpenResponsesInput] = None
    r"""Input for a response request - can be a string or array of items"""

    instructions: OptionalNullable[str] = UNSET

    metadata: OptionalNullable[Dict[str, str]] = UNSET
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""

    tools: Optional[List[OpenResponsesRequestToolUnion]] = None

    tool_choice: Optional[OpenAIResponsesToolChoiceUnion] = None

    parallel_tool_calls: OptionalNullable[bool] = UNSET

    model: Optional[str] = None

    models: Optional[List[str]] = None

    text: Optional[OpenResponsesResponseText] = None
    r"""Text output configuration including format and verbosity"""

    reasoning: OptionalNullable[OpenResponsesReasoningConfig] = UNSET
    r"""Configuration for reasoning mode in the response"""

    max_output_tokens: OptionalNullable[float] = UNSET

    temperature: OptionalNullable[float] = UNSET

    top_p: OptionalNullable[float] = UNSET

    top_k: Optional[float] = None

    prompt_cache_key: OptionalNullable[str] = UNSET

    previous_response_id: OptionalNullable[str] = UNSET

    prompt: OptionalNullable[OpenAIResponsesPrompt] = UNSET

    include: OptionalNullable[
        List[
            Annotated[
                OpenAIResponsesIncludable, PlainValidator(validate_open_enum(False))
            ]
        ]
    ] = UNSET

    background: OptionalNullable[bool] = UNSET

    safety_identifier: OptionalNullable[str] = UNSET

    store: OptionalNullable[bool] = UNSET

    service_tier: Annotated[
        OptionalNullable[ServiceTier], PlainValidator(validate_open_enum(False))
    ] = UNSET

    truncation: Annotated[
        OptionalNullable[Truncation], PlainValidator(validate_open_enum(False))
    ] = UNSET

    stream: Optional[bool] = False

    provider: OptionalNullable[Provider] = UNSET
    r"""When multiple model providers are available, optionally indicate your routing preference."""

    plugins: Optional[List[Plugin]] = None
    r"""Plugins you want to enable for this request, including their settings."""

    user: Optional[str] = None
    r"""A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "input",
            "instructions",
            "metadata",
            "tools",
            "tool_choice",
            "parallel_tool_calls",
            "model",
            "models",
            "text",
            "reasoning",
            "max_output_tokens",
            "temperature",
            "top_p",
            "top_k",
            "prompt_cache_key",
            "previous_response_id",
            "prompt",
            "include",
            "background",
            "safety_identifier",
            "store",
            "service_tier",
            "truncation",
            "stream",
            "provider",
            "plugins",
            "user",
        ]
        nullable_fields = [
            "instructions",
            "metadata",
            "parallel_tool_calls",
            "reasoning",
            "max_output_tokens",
            "temperature",
            "top_p",
            "prompt_cache_key",
            "previous_response_id",
            "prompt",
            "include",
            "background",
            "safety_identifier",
            "store",
            "service_tier",
            "truncation",
            "provider",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responsessearchcontextsize import ResponsesSearchContextSize
from .websearchpreviewtooluserlocation import (
    WebSearchPreviewToolUserLocation,
    WebSearchPreviewToolUserLocationTypedDict,
)
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


OpenResponsesWebSearchPreview20250311ToolType = Literal[
    "web_search_preview_2025_03_11",
]


class OpenResponsesWebSearchPreview20250311ToolTypedDict(TypedDict):
    r"""Web search preview tool configuration (2025-03-11 version)"""

    type: OpenResponsesWebSearchPreview20250311ToolType
    search_context_size: NotRequired[ResponsesSearchContextSize]
    r"""Size of the search context for web search tools"""
    user_location: NotRequired[Nullable[WebSearchPreviewToolUserLocationTypedDict]]


class OpenResponsesWebSearchPreview20250311Tool(BaseModel):
    r"""Web search preview tool configuration (2025-03-11 version)"""

    type: OpenResponsesWebSearchPreview20250311ToolType

    search_context_size: Annotated[
        Optional[ResponsesSearchContextSize], PlainValidator(validate_open_enum(False))
    ] = None
    r"""Size of the search context for web search tools"""

    user_location: OptionalNullable[WebSearchPreviewToolUserLocation] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["search_context_size", "user_location"]
        nullable_fields = ["user_location"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from openrouter.types import BaseModel
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ResponsesOutputMessageRole = Literal["assistant",]


ResponsesOutputMessageType = Literal["message",]


ResponsesOutputMessageStatusInProgress = Literal["in_progress",]


ResponsesOutputMessageStatusIncomplete = Literal["incomplete",]


ResponsesOutputMessageStatusCompleted = Literal["completed",]


ResponsesOutputMessageStatusUnionTypedDict = TypeAliasType(
    "ResponsesOutputMessageStatusUnionTypedDict",
    Union[
        ResponsesOutputMessageStatusCompleted,
        ResponsesOutputMessageStatusIncomplete,
        ResponsesOutputMessageStatusInProgress,
    ],
)


ResponsesOutputMessageStatusUnion = TypeAliasType(
    "ResponsesOutputMessageStatusUnion",
    Union[
        ResponsesOutputMessageStatusCompleted,
        ResponsesOutputMessageStatusIncomplete,
        ResponsesOutputMessageStatusInProgress,
    ],
)


ResponsesOutputMessageContentTypedDict = TypeAliasType(
    "ResponsesOutputMessageContentTypedDict",
    Union[OpenAIResponsesRefusalContentTypedDict, ResponseOutputTextTypedDict],
)


ResponsesOutputMessageContent = TypeAliasType(
    "ResponsesOutputMessageContent",
    Union[OpenAIResponsesRefusalContent, ResponseOutputText],
)


class ResponsesOutputMessageTypedDict(TypedDict):
    r"""An output message item"""

    id: str
    role: ResponsesOutputMessageRole
    type: ResponsesOutputMessageType
    content: List[ResponsesOutputMessageContentTypedDict]
    status: NotRequired[ResponsesOutputMessageStatusUnionTypedDict]


class ResponsesOutputMessage(BaseModel):
    r"""An output message item"""

    id: str

    role: ResponsesOutputMessageRole

    type: ResponsesOutputMessageType

    content: List[ResponsesOutputMessageContent]

    status: Optional[ResponsesOutputMessageStatusUnion] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, ClientOwner, HttpClient, close_clients
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
import httpx
import importlib
from openrouter import components, utils
from openrouter._hooks import SDKHooks
from openrouter.types import OptionalNullable, UNSET
import sys
from typing import Any, Callable, Dict, Optional, TYPE_CHECKING, Union, cast
import weakref

if TYPE_CHECKING:
    from openrouter.analytics import Analytics
    from openrouter.api_keys import APIKeys
    from openrouter.beta import Beta
    from openrouter.chat import Chat
    from openrouter.completions import Completions
    from openrouter.credits import Credits
    from openrouter.embeddings import Embeddings
    from openrouter.endpoints import Endpoints
    from openrouter.generations import Generations
    from openrouter.models_ import Models
    from openrouter.oauth import OAuth
    from openrouter.parameters import Parameters
    from openrouter.providers import Providers


class OpenRouter(BaseSDK):
    r"""OpenRouter API: OpenAI-compatible Chat Completions and Completions API with additional OpenRouter features
    https://openrouter.ai/docs - OpenRouter Documentation
    """

    beta: "Beta"
    analytics: "Analytics"
    r"""Analytics and usage endpoints"""
    credits: "Credits"
    r"""Credit management endpoints"""
    embeddings: "Embeddings"
    r"""Text embedding endpoints"""
    generations: "Generations"
    r"""Generation history endpoints"""
    models: "Models"
    r"""Model information endpoints"""
    endpoints: "Endpoints"
    r"""Endpoint information"""
    parameters: "Parameters"
    r"""Parameters endpoints"""
    providers: "Providers"
    r"""Provider information endpoints"""
    api_keys: "APIKeys"
    r"""API key management endpoints"""
    o_auth: "OAuth"
    r"""OAuth authentication endpoints"""
    chat: "Chat"
    completions: "Completions"
    _sub_sdk_map = {
        "beta": ("openrouter.beta", "Beta"),
        "analytics": ("openrouter.analytics", "Analytics"),
        "credits": ("openrouter.credits", "Credits"),
        "embeddings": ("openrouter.embeddings", "Embeddings"),
        "generations": ("openrouter.generations", "Generations"),
        "models": ("openrouter.models_", "Models"),
        "endpoints": ("openrouter.endpoints", "Endpoints"),
        "parameters": ("openrouter.parameters", "Parameters"),
        "providers": ("openrouter.providers", "Providers"),
        "api_keys": ("openrouter.api_keys", "APIKeys"),
        "o_auth": ("openrouter.oauth", "OAuth"),
        "chat": ("openrouter.chat", "Chat"),
        "completions": ("openrouter.completions", "Completions"),
    }

    def __init__(
        self,
        api_key: Optional[Union[Optional[str], Callable[[], Optional[str]]]] = None,
        server: Optional[str] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param api_key: The api_key required for authentication
        :param server: The server by name to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        client_supplied = True
        if client is None:
            client = httpx.Client(follow_redirects=True)
            client_supplied = False

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        async_client_supplied = True
        if async_client is None:
            async_client = httpx.AsyncClient(follow_redirects=True)
            async_client_supplied = False

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        security: Any = None
        if callable(api_key):
            # pylint: disable=unnecessary-lambda-assignment
            security = lambda: components.Security(api_key=api_key())
        else:
            security = components.Security(api_key=api_key)

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                client_supplied=client_supplied,
                async_client=async_client,
                async_client_supplied=async_client_supplied,
                security=security,
                server_url=server_url,
                server=server,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
            parent_ref=self,
        )

        hooks = SDKHooks()

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self.sdk_configuration = hooks.sdk_init(self.sdk_configuration)

        weakref.finalize(
            self,
            close_clients,
            cast(ClientOwner, self.sdk_configuration),
            self.sdk_configuration.client,
            self.sdk_configuration.client_supplied,
            self.sdk_configuration.async_client,
            self.sdk_configuration.async_client_supplied,
        )

    def dynamic_import(self, modname, retries=3):
        for attempt in range(retries):
            try:
                return importlib.import_module(modname)
            except KeyError:
                # Clear any half-initialized module and retry
                sys.modules.pop(modname, None)
                if attempt == retries - 1:
                    break
        raise KeyError(f"Failed to import module '{modname}' after {retries} attempts")

    def __getattr__(self, name: str):
        if name in self._sub_sdk_map:
            module_path, class_name = self._sub_sdk_map[name]
            try:
                module = self.dynamic_import(module_path)
                klass = getattr(module, class_name)
                instance = klass(self.sdk_configuration, parent_ref=self)
                setattr(self, name, instance)
                return instance
            except ImportError as e:
                raise AttributeError(
                    f"Failed to import module {module_path} for attribute {name}: {e}"
                ) from e
            except AttributeError as e:
                raise AttributeError(
                    f"Failed to find class {class_name} in module {module_path} for attribute {name}: {e}"
                ) from e

        raise AttributeError(
            f"'{type(self).__name__}' object has no attribute '{name}'"
        )

    def __dir__(self):
        default_attrs = list(super().__dir__())
        lazy_attrs = list(self._sub_sdk_map.keys())
        return sorted(list(set(default_attrs + lazy_attrs)))

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.client is not None
            and not self.sdk_configuration.client_supplied
        ):
            self.sdk_configuration.client.close()
        self.sdk_configuration.client = None

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if (
            self.sdk_configuration.async_client is not None
            and not self.sdk_configuration.async_client_supplied
        ):
            await self.sdk_configuration.async_client.aclose()
        self.sdk_configuration.async_client = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ReasoningTextContentType = Literal["reasoning_text",]


class ReasoningTextContentTypedDict(TypedDict):
    type: ReasoningTextContentType
    text: str


class ReasoningTextContent(BaseModel):
    type: ReasoningTextContentType

    text: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .jsonschemaconfig import JSONSchemaConfig, JSONSchemaConfigTypedDict
from openrouter.types import BaseModel
from openrouter.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ResponseFormatJSONSchemaTypedDict(TypedDict):
    json_schema: JSONSchemaConfigTypedDict
    type: Literal["json_schema"]


class ResponseFormatJSONSchema(BaseModel):
    json_schema: JSONSchemaConfig

    TYPE: Annotated[
        Annotated[
            Literal["json_schema"], AfterValidator(validate_const("json_schema"))
        ],
        pydantic.Field(alias="type"),
    ] = "json_schema"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesannotation import (
    OpenAIResponsesAnnotation,
    OpenAIResponsesAnnotationTypedDict,
)
from .openairesponsesrefusalcontent import (
    OpenAIResponsesRefusalContent,
    OpenAIResponsesRefusalContentTypedDict,
)
from .openresponseserrorevent import (
    OpenResponsesErrorEvent,
    OpenResponsesErrorEventTypedDict,
)
from .openresponsesimagegencallcompleted import (
    OpenResponsesImageGenCallCompleted,
    OpenResponsesImageGenCallCompletedTypedDict,
)
from .openresponsesimagegencallgenerating import (
    OpenResponsesImageGenCallGenerating,
    OpenResponsesImageGenCallGeneratingTypedDict,
)
from .openresponsesimagegencallinprogress import (
    OpenResponsesImageGenCallInProgress,
    OpenResponsesImageGenCallInProgressTypedDict,
)
from .openresponsesimagegencallpartialimage import (
    OpenResponsesImageGenCallPartialImage,
    OpenResponsesImageGenCallPartialImageTypedDict,
)
from .openresponseslogprobs import OpenResponsesLogProbs, OpenResponsesLogProbsTypedDict
from .openresponsesnonstreamingresponse import (
    OpenResponsesNonStreamingResponse,
    OpenResponsesNonStreamingResponseTypedDict,
)
from .openresponsesreasoningdeltaevent import (
    OpenResponsesReasoningDeltaEvent,
    OpenResponsesReasoningDeltaEventTypedDict,
)
from .openresponsesreasoningdoneevent import (
    OpenResponsesReasoningDoneEvent,
    OpenResponsesReasoningDoneEventTypedDict,
)
from .openresponsesreasoningsummarypartaddedevent import (
    OpenResponsesReasoningSummaryPartAddedEvent,
    OpenResponsesReasoningSummaryPartAddedEventTypedDict,
)
from .openresponsesreasoningsummarytextdeltaevent import (
    OpenResponsesReasoningSummaryTextDeltaEvent,
    OpenResponsesReasoningSummaryTextDeltaEventTypedDict,
)
from .openresponsesreasoningsummarytextdoneevent import (
    OpenResponsesReasoningSummaryTextDoneEvent,
    OpenResponsesReasoningSummaryTextDoneEventTypedDict,
)
from .reasoningsummarytext import ReasoningSummaryText, ReasoningSummaryTextTypedDict
from .reasoningtextcontent import ReasoningTextContent, ReasoningTextContentTypedDict
from .responseoutputtext import ResponseOutputText, ResponseOutputTextTypedDict
from .responsesoutputitem import ResponsesOutputItem, ResponsesOutputItemTypedDict
from openrouter.types import BaseModel
from typing import List, Literal, Union
from typing_extensions import TypeAliasType, TypedDict


TypeResponseReasoningSummaryPartDone = Literal["response.reasoning_summary_part.done",]


class OpenResponsesStreamEventResponseReasoningSummaryPartDoneTypedDict(TypedDict):
    r"""Event emitted when a reasoning summary part is complete"""

    type: TypeResponseReasoningSummaryPartDone
    output_index: float
    item_id: str
    summary_index: float
    part: ReasoningSummaryTextTypedDict
    sequence_number: float


class OpenResponsesStreamEventResponseReasoningSummaryPartDone(BaseModel):
    r"""Event emitted when a reasoning summary part is complete"""

    type: TypeResponseReasoningSummaryPartDone

    output_index: float

    item_id: str

    summary_index: float

    part: ReasoningSummaryText

    sequence_number: float


TypeResponseFunctionCallArgumentsDone = Literal[
    "response.function_call_arguments.done",
]


class OpenResponsesStreamEventResponseFunctionCallArgumentsDoneTypedDict(TypedDict):
    r"""Event emitted when function call arguments streaming is complete"""

    type: TypeResponseFunctionCallArgumentsDone
    item_id: str
    output_index: float
    name: str
    arguments: str
    sequence_number: float


class OpenResponsesStreamEventResponseFunctionCallArgumentsDone(BaseModel):
    r"""Event emitted when function call arguments streaming is complete"""

    type: TypeResponseFunctionCallArgumentsDone

    item_id: str

    output_index: float

    name: str

    arguments: str

    sequence_number: float


TypeResponseFunctionCallArgumentsDelta = Literal[
    "response.function_call_arguments.delta",
]


class OpenResponsesStreamEventResponseFunctionCallArgumentsDeltaTypedDict(TypedDict):
    r"""Event emitted when function call arguments are being streamed"""

    type: TypeResponseFunctionCallArgumentsDelta
    item_id: str
    output_index: float
    delta: str
    sequence_number: float


class OpenResponsesStreamEventResponseFunctionCallArgumentsDelta(BaseModel):
    r"""Event emitted when function call arguments are being streamed"""

    type: TypeResponseFunctionCallArgumentsDelta

    item_id: str

    output_index: float

    delta: str

    sequence_number: float


TypeResponseOutputTextAnnotationAdded = Literal[
    "response.output_text.annotation.added",
]


class OpenResponsesStreamEventResponseOutputTextAnnotationAddedTypedDict(TypedDict):
    r"""Event emitted when a text annotation is added to output"""

    type: TypeResponseOutputTextAnnotationAdded
    output_index: float
    item_id: str
    content_index: float
    sequence_number: float
    annotation_index: float
    annotation: OpenAIResponsesAnnotationTypedDict


class OpenResponsesStreamEventResponseOutputTextAnnotationAdded(BaseModel):
    r"""Event emitted when a text annotation is added to output"""

    type: TypeResponseOutputTextAnnotationAdded

    output_index: float

    item_id: str

    content_index: float

    sequence_number: float

    annotation_index: float

    annotation: OpenAIResponsesAnnotation


TypeResponseRefusalDone = Literal["response.refusal.done",]


class OpenResponsesStreamEventResponseRefusalDoneTypedDict(TypedDict):
    r"""Event emitted when refusal streaming is complete"""

    type: TypeResponseRefusalDone
    output_index: float
    item_id: str
    content_index: float
    refusal: str
    sequence_number: float


class OpenResponsesStreamEventResponseRefusalDone(BaseModel):
    r"""Event emitted when refusal streaming is complete"""

    type: TypeResponseRefusalDone

    output_index: float

    item_id: str

    content_index: float

    refusal: str

    sequence_number: float


TypeResponseRefusalDelta = Literal["response.refusal.delta",]


class OpenResponsesStreamEventResponseRefusalDeltaTypedDict(TypedDict):
    r"""Event emitted when a refusal delta is streamed"""

    type: TypeResponseRefusalDelta
    output_index: float
    item_id: str
    content_index: float
    delta: str
    sequence_number: float


class OpenResponsesStreamEventResponseRefusalDelta(BaseModel):
    r"""Event emitted when a refusal delta is streamed"""

    type: TypeResponseRefusalDelta

    output_index: float

    item_id: str

    content_index: float

    delta: str

    sequence_number: float


TypeResponseOutputTextDone = Literal["response.output_text.done",]


class OpenResponsesStreamEventResponseOutputTextDoneTypedDict(TypedDict):
    r"""Event emitted when text streaming is complete"""

    type: TypeResponseOutputTextDone
    output_index: float
    item_id: str
    content_index: float
    text: str
    sequence_number: float
    logprobs: List[OpenResponsesLogProbsTypedDict]


class OpenResponsesStreamEventResponseOutputTextDone(BaseModel):
    r"""Event emitted when text streaming is complete"""

    type: TypeResponseOutputTextDone

    output_index: float

    item_id: str

    content_index: float

    text: str

    sequence_number: float

    logprobs: List[OpenResponsesLogProbs]


TypeResponseOutputTextDelta = Literal["response.output_text.delta",]


class OpenResponsesStreamEventResponseOutputTextDeltaTypedDict(TypedDict):
    r"""Event emitted when a text delta is streamed"""

    type: TypeResponseOutputTextDelta
    logprobs: List[OpenResponsesLogProbsTypedDict]
    output_index: float
    item_id: str
    content_index: float
    delta: str
    sequence_number: float


class OpenResponsesStreamEventResponseOutputTextDelta(BaseModel):
    r"""Event emitted when a text delta is streamed"""

    type: TypeResponseOutputTextDelta

    logprobs: List[OpenResponsesLogProbs]

    output_index: float

    item_id: str

    content_index: float

    delta: str

    sequence_number: float


TypeResponseContentPartDone = Literal["response.content_part.done",]


Part2TypedDict = TypeAliasType(
    "Part2TypedDict",
    Union[
        ReasoningTextContentTypedDict,
        OpenAIResponsesRefusalContentTypedDict,
        ResponseOutputTextTypedDict,
    ],
)


Part2 = TypeAliasType(
    "Part2",
    Union[ReasoningTextContent, OpenAIResponsesRefusalContent, ResponseOutputText],
)


class OpenResponsesStreamEventResponseContentPartDoneTypedDict(TypedDict):
    r"""Event emitted when a content part is complete"""

    type: TypeResponseContentPartDone
    output_index: float
    item_id: str
    content_index: float
    part: Part2TypedDict
    sequence_number: float


class OpenResponsesStreamEventResponseContentPartDone(BaseModel):
    r"""Event emitted when a content part is complete"""

    type: TypeResponseContentPartDone

    output_index: float

    item_id: str

    content_index: float

    part: Part2

    sequence_number: float


TypeResponseContentPartAdded = Literal["response.content_part.added",]


Part1TypedDict = TypeAliasType(
    "Part1TypedDict",
    Union[
        ReasoningTextContentTypedDict,
        OpenAIResponsesRefusalContentTypedDict,
        ResponseOutputTextTypedDict,
    ],
)


Part1 = TypeAliasType(
    "Part1",
    Union[ReasoningTextContent, OpenAIResponsesRefusalContent, ResponseOutputText],
)


class OpenResponsesStreamEventResponseContentPartAddedTypedDict(TypedDict):
    r"""Event emitted when a new content part is added to an output item"""

    type: TypeResponseContentPartAdded
    output_index: float
    item_id: str
    content_index: float
    part: Part1TypedDict
    sequence_number: float


class OpenResponsesStreamEventResponseContentPartAdded(BaseModel):
    r"""Event emitted when a new content part is added to an output item"""

    type: TypeResponseContentPartAdded

    output_index: float

    item_id: str

    content_index: float

    part: Part1

    sequence_number: float


TypeResponseOutputItemDone = Literal["response.output_item.done",]


class OpenResponsesStreamEventResponseOutputItemDoneTypedDict(TypedDict):
    r"""Event emitted when an output item is complete"""

    type: TypeResponseOutputItemDone
    output_index: float
    item: ResponsesOutputItemTypedDict
    r"""An output item from the response"""
    sequence_number: float


class OpenResponsesStreamEventResponseOutputItemDone(BaseModel):
    r"""Event emitted when an output item is complete"""

    type: TypeResponseOutputItemDone

    output_index: float

    item: ResponsesOutputItem
    r"""An output item from the response"""

    sequence_number: float


TypeResponseOutputItemAdded = Literal["response.output_item.added",]


class OpenResponsesStreamEventResponseOutputItemAddedTypedDict(TypedDict):
    r"""Event emitted when a new output item is added to the response"""

    type: TypeResponseOutputItemAdded
    output_index: float
    item: ResponsesOutputItemTypedDict
    r"""An output item from the response"""
    sequence_number: float


class OpenResponsesStreamEventResponseOutputItemAdded(BaseModel):
    r"""Event emitted when a new output item is added to the response"""

    type: TypeResponseOutputItemAdded

    output_index: float

    item: ResponsesOutputItem
    r"""An output item from the response"""

    sequence_number: float


TypeResponseFailed = Literal["response.failed",]


class OpenResponsesStreamEventResponseFailedTypedDict(TypedDict):
    r"""Event emitted when a response has failed"""

    type: TypeResponseFailed
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseFailed(BaseModel):
    r"""Event emitted when a response has failed"""

    type: TypeResponseFailed

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseIncomplete = Literal["response.incomplete",]


class OpenResponsesStreamEventResponseIncompleteTypedDict(TypedDict):
    r"""Event emitted when a response is incomplete"""

    type: TypeResponseIncomplete
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseIncomplete(BaseModel):
    r"""Event emitted when a response is incomplete"""

    type: TypeResponseIncomplete

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseCompleted = Literal["response.completed",]


class OpenResponsesStreamEventResponseCompletedTypedDict(TypedDict):
    r"""Event emitted when a response has completed successfully"""

    type: TypeResponseCompleted
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseCompleted(BaseModel):
    r"""Event emitted when a response has completed successfully"""

    type: TypeResponseCompleted

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseInProgress = Literal["response.in_progress",]


class OpenResponsesStreamEventResponseInProgressTypedDict(TypedDict):
    r"""Event emitted when a response is in progress"""

    type: TypeResponseInProgress
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseInProgress(BaseModel):
    r"""Event emitted when a response is in progress"""

    type: TypeResponseInProgress

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


TypeResponseCreated = Literal["response.created",]


class OpenResponsesStreamEventResponseCreatedTypedDict(TypedDict):
    r"""Event emitted when a response is created"""

    type: TypeResponseCreated
    response: OpenResponsesNonStreamingResponseTypedDict
    r"""Complete non-streaming response from the Responses API"""
    sequence_number: float


class OpenResponsesStreamEventResponseCreated(BaseModel):
    r"""Event emitted when a response is created"""

    type: TypeResponseCreated

    response: OpenResponsesNonStreamingResponse
    r"""Complete non-streaming response from the Responses API"""

    sequence_number: float


OpenResponsesStreamEventTypedDict = TypeAliasType(
    "OpenResponsesStreamEventTypedDict",
    Union[
        OpenResponsesStreamEventResponseCreatedTypedDict,
        OpenResponsesStreamEventResponseInProgressTypedDict,
        OpenResponsesStreamEventResponseCompletedTypedDict,
        OpenResponsesStreamEventResponseIncompleteTypedDict,
        OpenResponsesStreamEventResponseFailedTypedDict,
        OpenResponsesStreamEventResponseOutputItemAddedTypedDict,
        OpenResponsesStreamEventResponseOutputItemDoneTypedDict,
        OpenResponsesImageGenCallCompletedTypedDict,
        OpenResponsesImageGenCallGeneratingTypedDict,
        OpenResponsesImageGenCallInProgressTypedDict,
        OpenResponsesErrorEventTypedDict,
        OpenResponsesStreamEventResponseFunctionCallArgumentsDeltaTypedDict,
        OpenResponsesStreamEventResponseRefusalDeltaTypedDict,
        OpenResponsesReasoningSummaryPartAddedEventTypedDict,
        OpenResponsesStreamEventResponseContentPartAddedTypedDict,
        OpenResponsesImageGenCallPartialImageTypedDict,
        OpenResponsesStreamEventResponseFunctionCallArgumentsDoneTypedDict,
        OpenResponsesReasoningDeltaEventTypedDict,
        OpenResponsesReasoningDoneEventTypedDict,
        OpenResponsesStreamEventResponseRefusalDoneTypedDict,
        OpenResponsesStreamEventResponseReasoningSummaryPartDoneTypedDict,
        OpenResponsesReasoningSummaryTextDeltaEventTypedDict,
        OpenResponsesReasoningSummaryTextDoneEventTypedDict,
        OpenResponsesStreamEventResponseContentPartDoneTypedDict,
        OpenResponsesStreamEventResponseOutputTextDeltaTypedDict,
        OpenResponsesStreamEventResponseOutputTextDoneTypedDict,
        OpenResponsesStreamEventResponseOutputTextAnnotationAddedTypedDict,
    ],
)
r"""Union of all possible event types emitted during response streaming"""


OpenResponsesStreamEvent = TypeAliasType(
    "OpenResponsesStreamEvent",
    Union[
        OpenResponsesStreamEventResponseCreated,
        OpenResponsesStreamEventResponseInProgress,
        OpenResponsesStreamEventResponseCompleted,
        OpenResponsesStreamEventResponseIncomplete,
        OpenResponsesStreamEventResponseFailed,
        OpenResponsesStreamEventResponseOutputItemAdded,
        OpenResponsesStreamEventResponseOutputItemDone,
        OpenResponsesImageGenCallCompleted,
        OpenResponsesImageGenCallGenerating,
        OpenResponsesImageGenCallInProgress,
        OpenResponsesErrorEvent,
        OpenResponsesStreamEventResponseFunctionCallArgumentsDelta,
        OpenResponsesStreamEventResponseRefusalDelta,
        OpenResponsesReasoningSummaryPartAddedEvent,
        OpenResponsesStreamEventResponseContentPartAdded,
        OpenResponsesImageGenCallPartialImage,
        OpenResponsesStreamEventResponseFunctionCallArgumentsDone,
        OpenResponsesReasoningDeltaEvent,
        OpenResponsesReasoningDoneEvent,
        OpenResponsesStreamEventResponseRefusalDone,
        OpenResponsesStreamEventResponseReasoningSummaryPartDone,
        OpenResponsesReasoningSummaryTextDeltaEvent,
        OpenResponsesReasoningSummaryTextDoneEvent,
        OpenResponsesStreamEventResponseContentPartDone,
        OpenResponsesStreamEventResponseOutputTextDelta,
        OpenResponsesStreamEventResponseOutputTextDone,
        OpenResponsesStreamEventResponseOutputTextAnnotationAdded,
    ],
)
r"""Union of all possible event types emitted during response streaming"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesannotation import (
    OpenAIResponsesAnnotation,
    OpenAIResponsesAnnotationTypedDict,
)
from openrouter.types import BaseModel
from typing import List, Literal, Optional
from typing_extensions import NotRequired, TypedDict


ResponseOutputTextType = Literal["output_text",]


class ResponseOutputTextTypedDict(TypedDict):
    type: ResponseOutputTextType
    text: str
    annotations: NotRequired[List[OpenAIResponsesAnnotationTypedDict]]


class ResponseOutputText(BaseModel):
    type: ResponseOutputTextType

    text: str

    annotations: Optional[List[OpenAIResponsesAnnotation]] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class VideoURLTypedDict(TypedDict):
    url: str


class VideoURL(BaseModel):
    url: str


class ChatMessageContentItemVideoTypedDict(TypedDict):
    video_url: VideoURLTypedDict
    type: Literal["input_video"]


class ChatMessageContentItemVideo(BaseModel):
    video_url: VideoURL

    TYPE: Annotated[
        Annotated[
            Literal["input_video"], AfterValidator(validate_const("input_video"))
        ],
        pydantic.Field(alias="type"),
    ] = "input_video"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_const, validate_open_enum
import pydantic
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


ChatMessageContentItemImageDetail = Union[
    Literal[
        "auto",
        "low",
        "high",
    ],
    UnrecognizedStr,
]


class ImageURLTypedDict(TypedDict):
    url: str
    detail: NotRequired[ChatMessageContentItemImageDetail]


class ImageURL(BaseModel):
    url: str

    detail: Annotated[
        Optional[ChatMessageContentItemImageDetail],
        PlainValidator(validate_open_enum(False)),
    ] = None


class ChatMessageContentItemImageTypedDict(TypedDict):
    image_url: ImageURLTypedDict
    type: Literal["image_url"]


class ChatMessageContentItemImage(BaseModel):
    image_url: ImageURL

    TYPE: Annotated[
        Annotated[Literal["image_url"], AfterValidator(validate_const("image_url"))],
        pydantic.Field(alias="type"),
    ] = "image_url"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_open_enum
import pydantic
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


ResponseInputAudioType = Literal["input_audio",]


ResponseInputAudioFormat = Union[
    Literal[
        "mp3",
        "wav",
    ],
    UnrecognizedStr,
]


class ResponseInputAudioInputAudioTypedDict(TypedDict):
    data: str
    format_: ResponseInputAudioFormat


class ResponseInputAudioInputAudio(BaseModel):
    data: str

    format_: Annotated[
        Annotated[ResponseInputAudioFormat, PlainValidator(validate_open_enum(False))],
        pydantic.Field(alias="format"),
    ]


class ResponseInputAudioTypedDict(TypedDict):
    r"""Audio input content item"""

    type: ResponseInputAudioType
    input_audio: ResponseInputAudioInputAudioTypedDict


class ResponseInputAudio(BaseModel):
    r"""Audio input content item"""

    type: ResponseInputAudioType

    input_audio: ResponseInputAudioInputAudio



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedInt
from typing import Literal, Union


EndpointStatus = Union[
    Literal[
        0,
        -1,
        -2,
        -3,
        -5,
        -10,
    ],
    UnrecognizedInt,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .reasoningsummarytext import ReasoningSummaryText, ReasoningSummaryTextTypedDict
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesReasoningSummaryPartAddedEventType = Literal[
    "response.reasoning_summary_part.added",
]


class OpenResponsesReasoningSummaryPartAddedEventTypedDict(TypedDict):
    r"""Event emitted when a reasoning summary part is added"""

    type: OpenResponsesReasoningSummaryPartAddedEventType
    output_index: float
    item_id: str
    summary_index: float
    part: ReasoningSummaryTextTypedDict
    sequence_number: float


class OpenResponsesReasoningSummaryPartAddedEvent(BaseModel):
    r"""Event emitted when a reasoning summary part is added"""

    type: OpenResponsesReasoningSummaryPartAddedEventType

    output_index: float

    item_id: str

    summary_index: float

    part: ReasoningSummaryText

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


URLCitationType = Literal["url_citation",]


class URLCitationTypedDict(TypedDict):
    type: URLCitationType
    url: str
    title: str
    start_index: float
    end_index: float


class URLCitation(BaseModel):
    type: URLCitationType

    url: str

    title: str

    start_index: float

    end_index: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatstreamoptions import ChatStreamOptions, ChatStreamOptionsTypedDict
from .message import Message, MessageTypedDict
from .reasoningsummaryverbosity import ReasoningSummaryVerbosity
from .responseformatjsonschema import (
    ResponseFormatJSONSchema,
    ResponseFormatJSONSchemaTypedDict,
)
from .responseformattextgrammar import (
    ResponseFormatTextGrammar,
    ResponseFormatTextGrammarTypedDict,
)
from .tooldefinitionjson import ToolDefinitionJSON, ToolDefinitionJSONTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_const, validate_open_enum
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


Effort = Union[
    Literal[
        "minimal",
        "low",
        "medium",
        "high",
    ],
    UnrecognizedStr,
]


class ReasoningTypedDict(TypedDict):
    effort: NotRequired[Nullable[Effort]]
    summary: NotRequired[Nullable[ReasoningSummaryVerbosity]]


class Reasoning(BaseModel):
    effort: Annotated[
        OptionalNullable[Effort], PlainValidator(validate_open_enum(False))
    ] = UNSET

    summary: Annotated[
        OptionalNullable[ReasoningSummaryVerbosity],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["effort", "summary"]
        nullable_fields = ["effort", "summary"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ChatGenerationParamsResponseFormatPythonTypedDict(TypedDict):
    type: Literal["python"]


class ChatGenerationParamsResponseFormatPython(BaseModel):
    TYPE: Annotated[
        Annotated[Literal["python"], AfterValidator(validate_const("python"))],
        pydantic.Field(alias="type"),
    ] = "python"


class ChatGenerationParamsResponseFormatJSONObjectTypedDict(TypedDict):
    type: Literal["json_object"]


class ChatGenerationParamsResponseFormatJSONObject(BaseModel):
    TYPE: Annotated[
        Annotated[
            Literal["json_object"], AfterValidator(validate_const("json_object"))
        ],
        pydantic.Field(alias="type"),
    ] = "json_object"


class ChatGenerationParamsResponseFormatTextTypedDict(TypedDict):
    type: Literal["text"]


class ChatGenerationParamsResponseFormatText(BaseModel):
    TYPE: Annotated[
        Annotated[Literal["text"], AfterValidator(validate_const("text"))],
        pydantic.Field(alias="type"),
    ] = "text"


ChatGenerationParamsResponseFormatUnionTypedDict = TypeAliasType(
    "ChatGenerationParamsResponseFormatUnionTypedDict",
    Union[
        ChatGenerationParamsResponseFormatTextTypedDict,
        ChatGenerationParamsResponseFormatJSONObjectTypedDict,
        ChatGenerationParamsResponseFormatPythonTypedDict,
        ResponseFormatJSONSchemaTypedDict,
        ResponseFormatTextGrammarTypedDict,
    ],
)


ChatGenerationParamsResponseFormatUnion = TypeAliasType(
    "ChatGenerationParamsResponseFormatUnion",
    Union[
        ChatGenerationParamsResponseFormatText,
        ChatGenerationParamsResponseFormatJSONObject,
        ChatGenerationParamsResponseFormatPython,
        ResponseFormatJSONSchema,
        ResponseFormatTextGrammar,
    ],
)


ChatGenerationParamsStopTypedDict = TypeAliasType(
    "ChatGenerationParamsStopTypedDict", Union[str, List[str]]
)


ChatGenerationParamsStop = TypeAliasType(
    "ChatGenerationParamsStop", Union[str, List[str]]
)


class ChatGenerationParamsTypedDict(TypedDict):
    messages: List[MessageTypedDict]
    model: NotRequired[str]
    models: NotRequired[List[str]]
    frequency_penalty: NotRequired[Nullable[float]]
    logit_bias: NotRequired[Nullable[Dict[str, float]]]
    logprobs: NotRequired[Nullable[bool]]
    top_logprobs: NotRequired[Nullable[float]]
    max_completion_tokens: NotRequired[Nullable[float]]
    max_tokens: NotRequired[Nullable[float]]
    metadata: NotRequired[Dict[str, str]]
    presence_penalty: NotRequired[Nullable[float]]
    reasoning: NotRequired[ReasoningTypedDict]
    response_format: NotRequired[ChatGenerationParamsResponseFormatUnionTypedDict]
    seed: NotRequired[Nullable[int]]
    stop: NotRequired[Nullable[ChatGenerationParamsStopTypedDict]]
    stream: NotRequired[bool]
    stream_options: NotRequired[Nullable[ChatStreamOptionsTypedDict]]
    temperature: NotRequired[Nullable[float]]
    tool_choice: NotRequired[Any]
    tools: NotRequired[List[ToolDefinitionJSONTypedDict]]
    top_p: NotRequired[Nullable[float]]
    user: NotRequired[str]


class ChatGenerationParams(BaseModel):
    messages: List[Message]

    model: Optional[str] = None

    models: Optional[List[str]] = None

    frequency_penalty: OptionalNullable[float] = UNSET

    logit_bias: OptionalNullable[Dict[str, float]] = UNSET

    logprobs: OptionalNullable[bool] = UNSET

    top_logprobs: OptionalNullable[float] = UNSET

    max_completion_tokens: OptionalNullable[float] = UNSET

    max_tokens: OptionalNullable[float] = UNSET

    metadata: Optional[Dict[str, str]] = None

    presence_penalty: OptionalNullable[float] = UNSET

    reasoning: Optional[Reasoning] = None

    response_format: Optional[ChatGenerationParamsResponseFormatUnion] = None

    seed: OptionalNullable[int] = UNSET

    stop: OptionalNullable[ChatGenerationParamsStop] = UNSET

    stream: Optional[bool] = False

    stream_options: OptionalNullable[ChatStreamOptions] = UNSET

    temperature: OptionalNullable[float] = UNSET

    tool_choice: Optional[Any] = None

    tools: Optional[List[ToolDefinitionJSON]] = None

    top_p: OptionalNullable[float] = UNSET

    user: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "model",
            "models",
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "top_logprobs",
            "max_completion_tokens",
            "max_tokens",
            "metadata",
            "presence_penalty",
            "reasoning",
            "response_format",
            "seed",
            "stop",
            "stream",
            "stream_options",
            "temperature",
            "tool_choice",
            "tools",
            "top_p",
            "user",
        ]
        nullable_fields = [
            "frequency_penalty",
            "logit_bias",
            "logprobs",
            "top_logprobs",
            "max_completion_tokens",
            "max_tokens",
            "presence_penalty",
            "seed",
            "stop",
            "stream_options",
            "temperature",
            "top_p",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responsesimagegenerationcall import (
    ResponsesImageGenerationCall,
    ResponsesImageGenerationCallTypedDict,
)
from .responsesoutputitemfilesearchcall import (
    ResponsesOutputItemFileSearchCall,
    ResponsesOutputItemFileSearchCallTypedDict,
)
from .responsesoutputitemfunctioncall import (
    ResponsesOutputItemFunctionCall,
    ResponsesOutputItemFunctionCallTypedDict,
)
from .responsesoutputitemreasoning import (
    ResponsesOutputItemReasoning,
    ResponsesOutputItemReasoningTypedDict,
)
from .responsesoutputmessage import (
    ResponsesOutputMessage,
    ResponsesOutputMessageTypedDict,
)
from .responseswebsearchcalloutput import (
    ResponsesWebSearchCallOutput,
    ResponsesWebSearchCallOutputTypedDict,
)
from typing import Union
from typing_extensions import TypeAliasType


ResponsesOutputItemTypedDict = TypeAliasType(
    "ResponsesOutputItemTypedDict",
    Union[
        ResponsesWebSearchCallOutputTypedDict,
        ResponsesOutputItemFileSearchCallTypedDict,
        ResponsesImageGenerationCallTypedDict,
        ResponsesOutputMessageTypedDict,
        ResponsesOutputItemReasoningTypedDict,
        ResponsesOutputItemFunctionCallTypedDict,
    ],
)
r"""An output item from the response"""


ResponsesOutputItem = TypeAliasType(
    "ResponsesOutputItem",
    Union[
        ResponsesWebSearchCallOutput,
        ResponsesOutputItemFileSearchCall,
        ResponsesImageGenerationCall,
        ResponsesOutputMessage,
        ResponsesOutputItemReasoning,
        ResponsesOutputItemFunctionCall,
    ],
)
r"""An output item from the response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filecitation import FileCitation, FileCitationTypedDict
from .filepath import FilePath, FilePathTypedDict
from .urlcitation import URLCitation, URLCitationTypedDict
from typing import Union
from typing_extensions import TypeAliasType


OpenAIResponsesAnnotationTypedDict = TypeAliasType(
    "OpenAIResponsesAnnotationTypedDict",
    Union[FilePathTypedDict, FileCitationTypedDict, URLCitationTypedDict],
)


OpenAIResponsesAnnotation = TypeAliasType(
    "OpenAIResponsesAnnotation", Union[FilePath, FileCitation, URLCitation]
)



"""Generate SHA-256 code challenge for PKCE OAuth flow"""

import base64
import hashlib
import re
import secrets
from dataclasses import dataclass
from typing import Optional


@dataclass
class CreateSHA256CodeChallengeRequest:
    """
    Request parameters for creating a SHA-256 code challenge.

    If not provided, a random code verifier will be generated.
    If provided, must be 43-128 characters and contain only unreserved
    characters [A-Za-z0-9-._~] per RFC 7636.
    """
    code_verifier: Optional[str] = None


@dataclass
class CreateSHA256CodeChallengeResponse:
    """Response containing the code challenge and verifier"""
    code_challenge: str
    code_verifier: str


def _array_buffer_to_base64_url(data: bytes) -> str:
    """
    Convert bytes to base64url encoding (RFC 4648)

    Args:
        data: Bytes to encode

    Returns:
        Base64url encoded string
    """
    return base64.urlsafe_b64encode(data).rstrip(b"=").decode("ascii")


def _generate_code_verifier() -> str:
    """
    Generate a cryptographically random code verifier per RFC 7636

    RFC 7636 recommends 32 octets of random data, base64url encoded = 43 chars

    Returns:
        A random code verifier string
    """
    random_bytes = secrets.token_bytes(32)
    return _array_buffer_to_base64_url(random_bytes)


def _validate_code_verifier(code_verifier: str) -> None:
    """
    Validate code verifier according to RFC 7636

    Args:
        code_verifier: The code verifier to validate

    Raises:
        ValueError: If the code verifier is invalid
    """
    if len(code_verifier) < 43:
        raise ValueError("Code verifier must be at least 43 characters")
    if len(code_verifier) > 128:
        raise ValueError("Code verifier must be at most 128 characters")
    if not re.match(r"^[A-Za-z0-9\-._~]+$", code_verifier):
        raise ValueError(
            "Code verifier must only contain unreserved characters: [A-Za-z0-9-._~]"
        )


def oauth_create_sha256_code_challenge(
    params: Optional[CreateSHA256CodeChallengeRequest] = None,
) -> CreateSHA256CodeChallengeResponse:
    """
    Generate a SHA-256 code challenge for PKCE

    Generates a SHA-256 code challenge and corresponding code verifier for use
    in the PKCE extension to OAuth2. If no code verifier is provided, a random
    one will be generated according to RFC 7636 (32 random bytes, base64url
    encoded). If a code verifier is provided, it must be 43-128 characters and
    contain only unreserved characters [A-Za-z0-9-._~].

    Args:
        params: Optional request parameters. If None, a random code verifier will be generated.

    Returns:
        CreateSHA256CodeChallengeResponse containing the code challenge and verifier

    Raises:
        ValueError: If the provided code verifier is invalid

    See Also:
        - https://openrouter.ai/docs/use-cases/oauth-pkce
        - https://datatracker.ietf.org/doc/html/rfc7636
    """
    if params is None:
        params = CreateSHA256CodeChallengeRequest()

    code_verifier = params.code_verifier
    if code_verifier is None:
        code_verifier = _generate_code_verifier()
    else:
        _validate_code_verifier(code_verifier)

    # Generate SHA-256 hash
    data = code_verifier.encode("utf-8")
    hash_digest = hashlib.sha256(data).digest()

    # Convert hash to base64url
    code_challenge = _array_buffer_to_base64_url(hash_digest)

    return CreateSHA256CodeChallengeResponse(
        code_challenge=code_challenge,
        code_verifier=code_verifier,
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import validate_const, validate_open_enum
import pydantic
from pydantic.functional_validators import AfterValidator, PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


ChatMessageContentItemAudioFormat = Union[
    Literal[
        "wav",
        "mp3",
        "flac",
        "m4a",
        "ogg",
        "pcm16",
        "pcm24",
    ],
    UnrecognizedStr,
]


class ChatMessageContentItemAudioInputAudioTypedDict(TypedDict):
    data: str
    format_: ChatMessageContentItemAudioFormat


class ChatMessageContentItemAudioInputAudio(BaseModel):
    data: str

    format_: Annotated[
        Annotated[
            ChatMessageContentItemAudioFormat, PlainValidator(validate_open_enum(False))
        ],
        pydantic.Field(alias="format"),
    ]


class ChatMessageContentItemAudioTypedDict(TypedDict):
    input_audio: ChatMessageContentItemAudioInputAudioTypedDict
    type: Literal["input_audio"]


class ChatMessageContentItemAudio(BaseModel):
    input_audio: ChatMessageContentItemAudioInputAudio

    TYPE: Annotated[
        Annotated[
            Literal["input_audio"], AfterValidator(validate_const("input_audio"))
        ],
        pydantic.Field(alias="type"),
    ] = "input_audio"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ResponseInputTextType = Literal["input_text",]


class ResponseInputTextTypedDict(TypedDict):
    r"""Text input content item"""

    type: ResponseInputTextType
    text: str


class ResponseInputText(BaseModel):
    r"""Text input content item"""

    type: ResponseInputTextType

    text: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel, UnrecognizedStr
from openrouter.utils import (
    FieldMetadata,
    PathParamMetadata,
    QueryParamMetadata,
    SecurityMetadata,
    validate_open_enum,
)
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypedDict


class GetParametersSecurityTypedDict(TypedDict):
    bearer: str


class GetParametersSecurity(BaseModel):
    bearer: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


GetParametersProvider = Union[
    Literal[
        "AI21",
        "AionLabs",
        "Alibaba",
        "Amazon Bedrock",
        "Anthropic",
        "AtlasCloud",
        "Atoma",
        "Avian",
        "Azure",
        "BaseTen",
        "Cerebras",
        "Chutes",
        "Cirrascale",
        "Clarifai",
        "Cloudflare",
        "Cohere",
        "CrofAI",
        "Crusoe",
        "DeepInfra",
        "DeepSeek",
        "Enfer",
        "Featherless",
        "Fireworks",
        "Friendli",
        "GMICloud",
        "Google",
        "Google AI Studio",
        "Groq",
        "Hyperbolic",
        "Inception",
        "InferenceNet",
        "Infermatic",
        "Inflection",
        "Kluster",
        "Lambda",
        "Liquid",
        "Mancer 2",
        "Meta",
        "Minimax",
        "ModelRun",
        "Mistral",
        "Modular",
        "Moonshot AI",
        "Morph",
        "NCompass",
        "Nebius",
        "NextBit",
        "Nineteen",
        "Novita",
        "Nvidia",
        "OpenAI",
        "OpenInference",
        "Parasail",
        "Perplexity",
        "Phala",
        "Relace",
        "SambaNova",
        "SiliconFlow",
        "Stealth",
        "Switchpoint",
        "Targon",
        "Together",
        "Ubicloud",
        "Venice",
        "WandB",
        "xAI",
        "Z.AI",
        "FakeProvider",
    ],
    UnrecognizedStr,
]


class GetParametersRequestTypedDict(TypedDict):
    author: str
    slug: str
    provider: NotRequired[GetParametersProvider]


class GetParametersRequest(BaseModel):
    author: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    slug: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    provider: Annotated[
        Annotated[
            Optional[GetParametersProvider], PlainValidator(validate_open_enum(False))
        ],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


SupportedParameter = Union[
    Literal[
        "temperature",
        "top_p",
        "top_k",
        "min_p",
        "top_a",
        "frequency_penalty",
        "presence_penalty",
        "repetition_penalty",
        "max_tokens",
        "logit_bias",
        "logprobs",
        "top_logprobs",
        "seed",
        "response_format",
        "structured_outputs",
        "stop",
        "tools",
        "tool_choice",
        "parallel_tool_calls",
        "include_reasoning",
        "reasoning",
        "web_search_options",
        "verbosity",
    ],
    UnrecognizedStr,
]


class GetParametersDataTypedDict(TypedDict):
    r"""Parameter analytics data"""

    model: str
    r"""Model identifier"""
    supported_parameters: List[SupportedParameter]
    r"""List of parameters supported by this model"""


class GetParametersData(BaseModel):
    r"""Parameter analytics data"""

    model: str
    r"""Model identifier"""

    supported_parameters: List[
        Annotated[SupportedParameter, PlainValidator(validate_open_enum(False))]
    ]
    r"""List of parameters supported by this model"""


class GetParametersResponseTypedDict(TypedDict):
    r"""Returns the parameters for the specified model"""

    data: GetParametersDataTypedDict
    r"""Parameter analytics data"""


class GetParametersResponse(BaseModel):
    r"""Returns the parameters for the specified model"""

    data: GetParametersData
    r"""Parameter analytics data"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ReasoningSummaryTextType = Literal["summary_text",]


class ReasoningSummaryTextTypedDict(TypedDict):
    type: ReasoningSummaryTextType
    text: str


class ReasoningSummaryText(BaseModel):
    type: ReasoningSummaryTextType

    text: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


FileCitationType = Literal["file_citation",]


class FileCitationTypedDict(TypedDict):
    type: FileCitationType
    file_id: str
    filename: str
    index: float


class FileCitation(BaseModel):
    type: FileCitationType

    file_id: str

    filename: str

    index: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


ChatCompletionFinishReason = Union[
    Literal[
        "tool_calls",
        "stop",
        "length",
        "content_filter",
        "error",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import (
    providername as components_providername,
    quantization as components_quantization,
)
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


InputTypedDict = TypeAliasType(
    "InputTypedDict", Union[str, List[str], List[float], List[List[float]]]
)


Input = TypeAliasType("Input", Union[str, List[str], List[float], List[List[float]]])


DataCollection = Union[
    Literal[
        "deny",
        "allow",
    ],
    UnrecognizedStr,
]
r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
- allow: (default) allow providers which store user data non-transiently and may train on it
- deny: use only providers which do not collect user data.

"""


OrderTypedDict = TypeAliasType(
    "OrderTypedDict", Union[components_providername.ProviderName, str]
)


Order = TypeAliasType(
    "Order",
    Union[
        Annotated[
            components_providername.ProviderName,
            PlainValidator(validate_open_enum(False)),
        ],
        str,
    ],
)


OnlyTypedDict = TypeAliasType(
    "OnlyTypedDict", Union[components_providername.ProviderName, str]
)


Only = TypeAliasType(
    "Only",
    Union[
        Annotated[
            components_providername.ProviderName,
            PlainValidator(validate_open_enum(False)),
        ],
        str,
    ],
)


IgnoreTypedDict = TypeAliasType(
    "IgnoreTypedDict", Union[components_providername.ProviderName, str]
)


Ignore = TypeAliasType(
    "Ignore",
    Union[
        Annotated[
            components_providername.ProviderName,
            PlainValidator(validate_open_enum(False)),
        ],
        str,
    ],
)


Sort = Union[
    Literal[
        "price",
        "throughput",
        "latency",
    ],
    UnrecognizedStr,
]
r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""


class MaxPriceTypedDict(TypedDict):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    completion: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    image: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    audio: NotRequired[Any]
    r"""A value in string or number format that is a large number"""
    request: NotRequired[Any]
    r"""A value in string or number format that is a large number"""


class MaxPrice(BaseModel):
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    prompt: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    completion: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    image: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    audio: Optional[Any] = None
    r"""A value in string or number format that is a large number"""

    request: Optional[Any] = None
    r"""A value in string or number format that is a large number"""


class CreateEmbeddingsProviderTypedDict(TypedDict):
    allow_fallbacks: NotRequired[Nullable[bool]]
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """
    require_parameters: NotRequired[Nullable[bool]]
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""
    data_collection: NotRequired[Nullable[DataCollection]]
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it
    - deny: use only providers which do not collect user data.

    """
    zdr: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""
    enforce_distillable_text: NotRequired[Nullable[bool]]
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""
    order: NotRequired[Nullable[List[OrderTypedDict]]]
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""
    only: NotRequired[Nullable[List[OnlyTypedDict]]]
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""
    ignore: NotRequired[Nullable[List[IgnoreTypedDict]]]
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""
    quantizations: NotRequired[Nullable[List[components_quantization.Quantization]]]
    r"""A list of quantization levels to filter the provider by."""
    sort: NotRequired[Nullable[Sort]]
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""
    max_price: NotRequired[MaxPriceTypedDict]
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""


class CreateEmbeddingsProvider(BaseModel):
    allow_fallbacks: OptionalNullable[bool] = UNSET
    r"""Whether to allow backup providers to serve requests
    - true: (default) when the primary provider (or your custom providers in \"order\") is unavailable, use the next best provider.
    - false: use only the primary/custom provider, and return the upstream error if it's unavailable.

    """

    require_parameters: OptionalNullable[bool] = UNSET
    r"""Whether to filter providers to only those that support the parameters you've provided. If this setting is omitted or set to false, then providers will receive only the parameters they support, and ignore the rest."""

    data_collection: Annotated[
        OptionalNullable[DataCollection], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""Data collection setting. If no available model provider meets the requirement, your request will return an error.
    - allow: (default) allow providers which store user data non-transiently and may train on it
    - deny: use only providers which do not collect user data.

    """

    zdr: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only ZDR (Zero Data Retention) endpoints. When true, only endpoints that do not retain prompts will be used."""

    enforce_distillable_text: OptionalNullable[bool] = UNSET
    r"""Whether to restrict routing to only models that allow text distillation. When true, only models where the author has allowed distillation will be used."""

    order: OptionalNullable[List[Order]] = UNSET
    r"""An ordered list of provider slugs. The router will attempt to use the first provider in the subset of this list that supports your requested model, and fall back to the next if it is unavailable. If no providers are available, the request will fail with an error message."""

    only: OptionalNullable[List[Only]] = UNSET
    r"""List of provider slugs to allow. If provided, this list is merged with your account-wide allowed provider settings for this request."""

    ignore: OptionalNullable[List[Ignore]] = UNSET
    r"""List of provider slugs to ignore. If provided, this list is merged with your account-wide ignored provider settings for this request."""

    quantizations: OptionalNullable[
        List[
            Annotated[
                components_quantization.Quantization,
                PlainValidator(validate_open_enum(False)),
            ]
        ]
    ] = UNSET
    r"""A list of quantization levels to filter the provider by."""

    sort: Annotated[
        OptionalNullable[Sort], PlainValidator(validate_open_enum(False))
    ] = UNSET
    r"""The sorting strategy to use for this request, if \"order\" is not specified. When set, no load balancing is performed."""

    max_price: Optional[MaxPrice] = None
    r"""The object specifying the maximum price you want to pay for this request. USD price per million tokens, for prompt and completion."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
            "max_price",
        ]
        nullable_fields = [
            "allow_fallbacks",
            "require_parameters",
            "data_collection",
            "zdr",
            "enforce_distillable_text",
            "order",
            "only",
            "ignore",
            "quantizations",
            "sort",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


EncodingFormatBase64 = Literal["base64",]


EncodingFormatFloat = Literal["float",]


EncodingFormatTypedDict = TypeAliasType(
    "EncodingFormatTypedDict", Union[EncodingFormatFloat, EncodingFormatBase64]
)


EncodingFormat = TypeAliasType(
    "EncodingFormat", Union[EncodingFormatFloat, EncodingFormatBase64]
)


class CreateEmbeddingsRequestTypedDict(TypedDict):
    input: InputTypedDict
    model: str
    provider: NotRequired[CreateEmbeddingsProviderTypedDict]
    encoding_format: NotRequired[EncodingFormatTypedDict]
    user: NotRequired[str]


class CreateEmbeddingsRequest(BaseModel):
    input: Input

    model: str

    provider: Optional[CreateEmbeddingsProvider] = None

    encoding_format: Optional[EncodingFormat] = None

    user: Optional[str] = None


Object = Literal["list",]


ObjectEmbedding = Literal["embedding",]


EmbeddingTypedDict = TypeAliasType("EmbeddingTypedDict", Union[List[float], str])


Embedding = TypeAliasType("Embedding", Union[List[float], str])


class CreateEmbeddingsDataTypedDict(TypedDict):
    object: ObjectEmbedding
    embedding: EmbeddingTypedDict
    index: NotRequired[float]


class CreateEmbeddingsData(BaseModel):
    object: ObjectEmbedding

    embedding: Embedding

    index: Optional[float] = None


class UsageTypedDict(TypedDict):
    prompt_tokens: float
    total_tokens: float
    cost: NotRequired[float]


class Usage(BaseModel):
    prompt_tokens: float

    total_tokens: float

    cost: Optional[float] = None


class CreateEmbeddingsResponseBodyTypedDict(TypedDict):
    r"""Embedding response"""

    object: Object
    data: List[CreateEmbeddingsDataTypedDict]
    model: str
    id: NotRequired[str]
    usage: NotRequired[UsageTypedDict]


class CreateEmbeddingsResponseBody(BaseModel):
    r"""Embedding response"""

    object: Object

    data: List[CreateEmbeddingsData]

    model: str

    id: Optional[str] = None

    usage: Optional[Usage] = None


CreateEmbeddingsResponseTypedDict = TypeAliasType(
    "CreateEmbeddingsResponseTypedDict",
    Union[CreateEmbeddingsResponseBodyTypedDict, str],
)


CreateEmbeddingsResponse = TypeAliasType(
    "CreateEmbeddingsResponse", Union[CreateEmbeddingsResponseBody, str]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union


class Parameters(BaseSDK):
    r"""Parameters endpoints"""

    def get_parameters(
        self,
        *,
        security: Union[
            operations.GetParametersSecurity, operations.GetParametersSecurityTypedDict
        ],
        author: str,
        slug: str,
        provider: Optional[operations.GetParametersProvider] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetParametersData:
        r"""Get a model's supported parameters and data about which are most popular

        :param security:
        :param author:
        :param slug:
        :param provider:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetParametersRequest(
            author=author,
            slug=slug,
            provider=provider,
        )

        req = self._build_request(
            method="GET",
            path="/parameters/{author}/{slug}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, operations.GetParametersSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getParameters",
                oauth2_scopes=None,
                security_source=get_security_from_env(security, components.Security),
            ),
            request=req,
            error_status_codes=["401", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetParametersResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def get_parameters_async(
        self,
        *,
        security: Union[
            operations.GetParametersSecurity, operations.GetParametersSecurityTypedDict
        ],
        author: str,
        slug: str,
        provider: Optional[operations.GetParametersProvider] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetParametersData:
        r"""Get a model's supported parameters and data about which are most popular

        :param security:
        :param author:
        :param slug:
        :param provider:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetParametersRequest(
            author=author,
            slug=slug,
            provider=provider,
        )

        req = self._build_request_async(
            method="GET",
            path="/parameters/{author}/{slug}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=utils.get_pydantic_model(
                security, operations.GetParametersSecurity
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getParameters",
                oauth2_scopes=None,
                security_source=get_security_from_env(security, components.Security),
            ),
            request=req,
            error_status_codes=["401", "404", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetParametersResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesReasoningDoneEventType = Literal["response.reasoning_text.done",]


class OpenResponsesReasoningDoneEventTypedDict(TypedDict):
    r"""Event emitted when reasoning text streaming is complete"""

    type: OpenResponsesReasoningDoneEventType
    output_index: float
    item_id: str
    content_index: float
    text: str
    sequence_number: float


class OpenResponsesReasoningDoneEvent(BaseModel):
    r"""Event emitted when reasoning text streaming is complete"""

    type: OpenResponsesReasoningDoneEventType

    output_index: float

    item_id: str

    content_index: float

    text: str

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


ResponsesFormatJSONObjectType = Literal["json_object",]


class ResponsesFormatJSONObjectTypedDict(TypedDict):
    r"""JSON object response format"""

    type: ResponsesFormatJSONObjectType


class ResponsesFormatJSONObject(BaseModel):
    r"""JSON object response format"""

    type: ResponsesFormatJSONObjectType



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


ModelGroup = Union[
    Literal[
        "Router",
        "Media",
        "Other",
        "GPT",
        "Claude",
        "Gemini",
        "Grok",
        "Cohere",
        "Nova",
        "Qwen",
        "Yi",
        "DeepSeek",
        "Mistral",
        "Llama2",
        "Llama3",
        "Llama4",
        "PaLM",
        "RWKV",
        "Qwen3",
    ],
    UnrecognizedStr,
]
r"""Tokenizer type used by the model"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .reasoningsummarytext import ReasoningSummaryText, ReasoningSummaryTextTypedDict
from .reasoningtextcontent import ReasoningTextContent, ReasoningTextContentTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Literal, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ResponsesOutputItemReasoningType = Literal["reasoning",]


ResponsesOutputItemReasoningStatusInProgress = Literal["in_progress",]


ResponsesOutputItemReasoningStatusIncomplete = Literal["incomplete",]


ResponsesOutputItemReasoningStatusCompleted = Literal["completed",]


ResponsesOutputItemReasoningStatusUnionTypedDict = TypeAliasType(
    "ResponsesOutputItemReasoningStatusUnionTypedDict",
    Union[
        ResponsesOutputItemReasoningStatusCompleted,
        ResponsesOutputItemReasoningStatusIncomplete,
        ResponsesOutputItemReasoningStatusInProgress,
    ],
)


ResponsesOutputItemReasoningStatusUnion = TypeAliasType(
    "ResponsesOutputItemReasoningStatusUnion",
    Union[
        ResponsesOutputItemReasoningStatusCompleted,
        ResponsesOutputItemReasoningStatusIncomplete,
        ResponsesOutputItemReasoningStatusInProgress,
    ],
)


class ResponsesOutputItemReasoningTypedDict(TypedDict):
    r"""An output item containing reasoning"""

    type: ResponsesOutputItemReasoningType
    id: str
    summary: List[ReasoningSummaryTextTypedDict]
    content: NotRequired[List[ReasoningTextContentTypedDict]]
    encrypted_content: NotRequired[Nullable[str]]
    status: NotRequired[ResponsesOutputItemReasoningStatusUnionTypedDict]


class ResponsesOutputItemReasoning(BaseModel):
    r"""An output item containing reasoning"""

    type: ResponsesOutputItemReasoningType

    id: str

    summary: List[ReasoningSummaryText]

    content: Optional[List[ReasoningTextContent]] = None

    encrypted_content: OptionalNullable[str] = UNSET

    status: Optional[ResponsesOutputItemReasoningStatusUnion] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["content", "encrypted_content", "status"]
        nullable_fields = ["encrypted_content"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openresponseseasyinputmessage import (
    OpenResponsesEasyInputMessage,
    OpenResponsesEasyInputMessageTypedDict,
)
from .openresponsesfunctioncalloutput import (
    OpenResponsesFunctionCallOutput,
    OpenResponsesFunctionCallOutputTypedDict,
)
from .openresponsesfunctiontoolcall import (
    OpenResponsesFunctionToolCall,
    OpenResponsesFunctionToolCallTypedDict,
)
from .openresponsesinputmessageitem import (
    OpenResponsesInputMessageItem,
    OpenResponsesInputMessageItemTypedDict,
)
from .openresponsesreasoning import (
    OpenResponsesReasoning,
    OpenResponsesReasoningTypedDict,
)
from .responsesimagegenerationcall import (
    ResponsesImageGenerationCall,
    ResponsesImageGenerationCallTypedDict,
)
from .responsesoutputitemfilesearchcall import (
    ResponsesOutputItemFileSearchCall,
    ResponsesOutputItemFileSearchCallTypedDict,
)
from .responsesoutputitemfunctioncall import (
    ResponsesOutputItemFunctionCall,
    ResponsesOutputItemFunctionCallTypedDict,
)
from .responsesoutputitemreasoning import (
    ResponsesOutputItemReasoning,
    ResponsesOutputItemReasoningTypedDict,
)
from .responsesoutputmessage import (
    ResponsesOutputMessage,
    ResponsesOutputMessageTypedDict,
)
from .responseswebsearchcalloutput import (
    ResponsesWebSearchCallOutput,
    ResponsesWebSearchCallOutputTypedDict,
)
from typing import List, Union
from typing_extensions import TypeAliasType


OpenResponsesInput1TypedDict = TypeAliasType(
    "OpenResponsesInput1TypedDict",
    Union[
        OpenResponsesEasyInputMessageTypedDict,
        ResponsesWebSearchCallOutputTypedDict,
        OpenResponsesInputMessageItemTypedDict,
        ResponsesOutputItemFileSearchCallTypedDict,
        ResponsesImageGenerationCallTypedDict,
        OpenResponsesFunctionCallOutputTypedDict,
        ResponsesOutputMessageTypedDict,
        OpenResponsesFunctionToolCallTypedDict,
        ResponsesOutputItemReasoningTypedDict,
        ResponsesOutputItemFunctionCallTypedDict,
        OpenResponsesReasoningTypedDict,
    ],
)


OpenResponsesInput1 = TypeAliasType(
    "OpenResponsesInput1",
    Union[
        OpenResponsesEasyInputMessage,
        ResponsesWebSearchCallOutput,
        OpenResponsesInputMessageItem,
        ResponsesOutputItemFileSearchCall,
        ResponsesImageGenerationCall,
        OpenResponsesFunctionCallOutput,
        ResponsesOutputMessage,
        OpenResponsesFunctionToolCall,
        ResponsesOutputItemReasoning,
        ResponsesOutputItemFunctionCall,
        OpenResponsesReasoning,
    ],
)


OpenResponsesInputTypedDict = TypeAliasType(
    "OpenResponsesInputTypedDict", Union[str, List[OpenResponsesInput1TypedDict]]
)
r"""Input for a response request - can be a string or array of items"""


OpenResponsesInput = TypeAliasType(
    "OpenResponsesInput", Union[str, List[OpenResponsesInput1]]
)
r"""Input for a response request - can be a string or array of items"""
